Package: colorspace
Version: 1.3-2
Date: 2016-12-14
Title: Color Space Manipulation
Authors@R: c(person(given = "Ross", family = "Ihaka", role = "aut", email = "ihaka@stat.auckland.ac.nz"),
             person(given = "Paul", family = "Murrell", role = "aut", email = "paul@stat.auckland.ac.nz"),
             person(given = "Kurt", family = "Hornik", role = "aut", email = "Kurt.Hornik@R-project.org"),
             person(given = c("Jason", "C."), family = "Fisher", role = "aut", email = "jfisher@usgs.gov"),
             person(given = "Reto", family = "Stauffer", role = "aut", email = "Reto.Stauffer@uibk.ac.at"),
             person(given = "Achim", family = "Zeileis", role = c("aut", "cre"), email = "Achim.Zeileis@R-project.org"))
Description: Carries out mapping between assorted color spaces including
             RGB, HSV, HLS, CIEXYZ, CIELUV, HCL (polar CIELUV),
	     CIELAB and polar CIELAB. Qualitative, sequential, and
	     diverging color palettes based on HCL colors are provided
	     along with an interactive palette picker (with either a Tcl/Tk
	     or a shiny GUI).
Depends: R (>= 2.13.0), methods
Imports: graphics, grDevices
Suggests: datasets, stats, utils, KernSmooth, MASS, kernlab, mvtnorm,
        vcd, dichromat, tcltk, shiny, shinyjs
License: BSD_3_clause + file LICENSE
URL: https://hclwizard.org/
LazyData: yes
NeedsCompilation: yes
Packaged: 2016-12-14 22:11:46 UTC; zeileis
Author: Ross Ihaka [aut],
  Paul Murrell [aut],
  Kurt Hornik [aut],
  Jason C. Fisher [aut],
  Reto Stauffer [aut],
  Achim Zeileis [aut, cre]
Maintainer: Achim Zeileis <Achim.Zeileis@R-project.org>
Repository: CRAN
Date/Publication: 2016-12-14 23:28:25
Built: R 3.4.1; x86_64-w64-mingw32; 2018-09-09 05:52:05 UTC; windows
Archs: x64
